@@ -18,10 +18,10 @@ from django.urls import path
 from . import views
 
 urlpatterns = [
-    path('login/', views.login, name='saml2_login'),
-    path('acs/', views.AssertionConsumerServiceView.as_view(), name='saml2_acs'),
-    path('logout/', views.logout, name='saml2_logout'),
-    path('ls/', views.logout_service, name='saml2_ls'),
-    path('ls/post/', views.logout_service_post, name='saml2_ls_post'),
-    path('metadata/', views.metadata, name='saml2_metadata'),
+    path("login/", views.LoginView.as_view(), name="saml2_login"),
+    path("acs/", views.AssertionConsumerServiceView.as_view(), name="saml2_acs"),
+    path("logout/", views.LogoutInitView.as_view(), name="saml2_logout"),
+    path("ls/", views.LogoutView.as_view(), name="saml2_ls"),
+    path("ls/post/", views.LogoutView.as_view(), name="saml2_ls_post"),
+    path("metadata/", views.MetadataView.as_view(), name="saml2_metadata"),
 ]
