@@ -15,6 +15,7 @@
 
 import base64
 import logging
+from functools import wraps
 from typing import Optional
 from urllib.parse import quote
 
@@ -69,6 +70,7 @@ from .overrides import Saml2Client
 from .utils import (
     add_idp_hinting,
     available_idps,
+    get_csp_handler,
     get_custom_setting,
     get_fallback_login_redirect_url,
     get_idp_sso_supported_bindings,
@@ -78,25 +80,15 @@ from .utils import (
 
 logger = logging.getLogger("djangosaml2")
 
-# Update Content-Security-Policy headers for POST-Bindings
-try:
-    from csp.decorators import csp_update
-except ModuleNotFoundError:
-    # If csp is not installed, do not update fields as Content-Security-Policy
-    # is not used
-    def saml2_csp_update(view):
-        return view
 
-    logger.warning("django-csp could not be found, not updating Content-Security-Policy. Please "
-                   "make sure CSP is configured at least by httpd or setup django-csp. See "
-                   "https://djangosaml2.readthedocs.io/contents/security.html#content-security-policy"
-                   " for more information")
-else:
-    # script-src 'unsafe-inline' to autosubmit forms,
-    # form-action https: to send data to IdPs
-    saml2_csp_update = csp_update(
-        SCRIPT_SRC=["'unsafe-inline'"], FORM_ACTION=["https:"]
-    )
+def saml2_csp_update(view):
+    csp_handler = get_csp_handler()
+
+    @wraps(view)
+    def wrapper(*args, **kwargs):
+        return csp_handler(view)(*args, **kwargs)
+
+    return wrapper
 
 
 def _set_subject_id(session, subject_id):
@@ -175,7 +167,7 @@ class LoginView(SPConfigMixin, View):
 
     def unknown_idp(self, request, idp):
         msg = f"Error: IdP EntityID {escape(idp)} was not found in metadata"
-        logger.error(msg)
+        logger.exception(msg)
         return HttpResponse(msg, status=403)
 
     def load_sso_kwargs_scoping(self, sso_kwargs):
@@ -358,7 +350,7 @@ class LoginView(SPConfigMixin, View):
                     **sso_kwargs,
                 )
             except TypeError as e:
-                logger.error(f"{_msg}: {e}")
+                logger.exception(f"{_msg}: {e}")
                 return HttpResponse(_msg)
             else:
                 http_response = HttpResponseRedirect(get_location(result))
@@ -369,7 +361,7 @@ class LoginView(SPConfigMixin, View):
                 try:
                     location = client.sso_location(selected_idp, binding)
                 except TypeError as e:
-                    logger.error(f"{_msg}: {e}")
+                    logger.exception(f"{_msg}: {e}")
                     return HttpResponse(_msg)
 
                 session_id, request_xml = client.create_authn_request(
@@ -396,7 +388,8 @@ class LoginView(SPConfigMixin, View):
                     )
                 except TemplateDoesNotExist as e:
                     logger.debug(
-                        f"TemplateDoesNotExist: [{self.post_binding_form_template}] - {e}"
+                        f"TemplateDoesNotExist: [{self.post_binding_form_template}] - {e}",
+                        exc_info=True
                     )
 
             if not http_response:
@@ -410,7 +403,7 @@ class LoginView(SPConfigMixin, View):
                     )
                 except TypeError as e:
                     _msg = f"Can't prepare the authentication for {selected_idp}"
-                    logger.error(f"{_msg}: {e}")
+                    logger.exception(f"{_msg}: {e}")
                     return HttpResponse(_msg)
                 else:
                     http_response = HttpResponse(result["data"])
@@ -545,7 +538,7 @@ class AssertionConsumerServiceView(SPConfigMixin, View):
         try:
             self.custom_validation(response)
         except Exception as e:
-            logger.warning(f"SAML Response validation error: {e}")
+            logger.warning(f"SAML Response validation error: {e}", exc_info=True)
             return self.handle_acs_failure(
                 request,
                 status=400,
@@ -795,7 +788,7 @@ class LogoutView(SPConfigMixin, View):
             request, request.POST, saml2.BINDING_HTTP_POST, *args, **kwargs
         )
 
-    def do_logout_service(self, request, data, binding):
+    def do_logout_service(self, request, data, binding, *args, **kwargs):
         logger.debug("Logout service started")
 
         state, client = self.get_state_client(request)
@@ -808,7 +801,7 @@ class LogoutView(SPConfigMixin, View):
                 )
             except StatusError as e:
                 response = None
-                logger.warning("Error logging out from remote provider: " + str(e))
+                logger.warning(f"Error logging out from remote provider: {e}", exc_info=True)
             state.sync()
             return finish_logout(request, response)
 
